@@ -1,5 +1,5 @@
-#ifndef INCLUDED_CHTIME_AND_MOD_H
-#define INCLUDED_CHTIME_AND_MOD_H
+#ifndef CH_TIME_AND_MOD_H
+#define CH_TIME_AND_MOD_H
 
 #define _GNU_SOURCE
 
@@ -13,31 +13,21 @@
 static struct timespec time_s[2];
 
 // 関数プロトタイプ
-static inline void ch_time_and_mod(const struct stat *stat_buf, const CPInfo *cpinfo, CPDD *cpdd) __attribute__((always_inline));
+static inline void ch_time_and_mod(const int *ito, const struct stat *stat_buf, const CPInfo *cpinfo, CPDD *cpdd) __attribute__((always_inline));
 
 /*******************************************************************************
 *******************************************************************************/
-static inline void ch_time_and_mod(const struct stat *stat_buf, const CPInfo *cpinfo, CPDD *cpdd)
+static inline void ch_time_and_mod(const int *ito, const struct stat *stat_buf, const CPInfo *cpinfo, CPDD *cpdd)
 {
 	errno = 0;
 
-	if(cpinfo->from_type == REGULAR)
+	if(fchmod(*ito, stat_buf->st_mode) == -1)
 	{
-		if(chmod(cpinfo->to, stat_buf->st_mode) == -1)
-		{
-			print_error("chmod", __FILE__, __LINE__, cpdd);
-		}
-		if(chown(cpinfo->to, stat_buf->st_uid, stat_buf->st_gid) == -1)
-		{
-			print_error("chown", __FILE__, __LINE__, cpdd);
-		}
+		print_error("chmod", __FILE__, __LINE__, cpdd);
 	}
-	else
+	if(fchown(*ito, stat_buf->st_uid, stat_buf->st_gid) == -1)
 	{
-		if(lchown(cpinfo->to, stat_buf->st_uid, stat_buf->st_gid) == -1)
-		{
-			print_error("lchown", __FILE__, __LINE__, cpdd);
-		}
+		print_error("chown", __FILE__, __LINE__, cpdd);
 	}
 
 	time_s[0].tv_sec = stat_buf->st_atime;
