@@ -35,8 +35,22 @@ SDir * get_settings_dir(void)
 	errno = 0;
 	if((sdir->current_fd = open(".", O_RDONLY | O_NOATIME)) == -1)
 	{
-		perror("get_settings_dir");
-		exit(EXIT_FAILURE);
+		if(errno == EPERM)
+		{
+			errno = 0;
+			if((sdir->current_fd = open(".", O_RDONLY)) == -1)
+			{
+				printf("errno is %d\n", errno);
+				perror("get_settings_dir");
+				exit(EXIT_FAILURE);
+			}
+		}
+		else
+		{
+			printf("errno is %d\n", errno);
+			perror("get_settings_dir");
+			exit(EXIT_FAILURE);
+		}
 	}
 
 	{
