@@ -18,7 +18,7 @@
 
 #include <errno.h>
 #include <fcntl.h>
-#include <linux/falloc.h>
+//#include <linux/falloc.h>
 #include <pthread.h>
 #include <sys/mman.h>
 #include <sys/stat.h>
@@ -95,6 +95,7 @@ static long psize = 0;
 static _Bool from_open;
 static _Bool from_stat;
 static _Bool to_open;
+//static _Bool filesystem_is_ext4;
 
 // ※注意。グローバル変数。
 char *write_now;
@@ -108,11 +109,12 @@ static void * write_thread(void *unknown);
 *******************************************************************************/
 _Bool read_write(const OPArg *oparg, CPInfo *cpinfo, SDir *sdir, CPDD *cpdd)
 {
-	struct stat stat_buf;
+	static struct stat stat_buf;
 
 	from_stat = false;
 	from_open = false;
 	to_open = false;
+//	filesystem_is_ext4 = false;
 	from_size_local = 0;
 	written_size[0] = 0;
 	written_size[1] = 0;
@@ -214,21 +216,26 @@ _Bool read_write(const OPArg *oparg, CPInfo *cpinfo, SDir *sdir, CPDD *cpdd)
 	 * コピー前にファイルをトランケート。いわゆるsparse file化。
 	 * Windowsだと断片化防止に効果があるのだが、Linuxだと？
 	 * 
-	 * 1.2TBほどコピーしてみたあと、fsckを実行したら、
+	 * 追記。1.2TBほどコピーしてみたあと、fsckを実行したら、
 	 * non-contiguousは3.2%だった。効果無し？
 	 * 
-	 * ↑と同じデータをトランケート無しでコピーしてみたら、
+	 * 追記。↑と同じデータをトランケート無しでコピーしてみたら、
 	 * non-contiguousの値は同じだった。
 	 * non-contiguousって信頼していいのか？
 	 * 
-	 * NTFSやFATに書き込む前にftruncateすると、
+	 * 追記。NTFSやFATに書き込む前にftruncateすると、
 	 * 断片化を押さえることが出来ることを確認。
 	 * 
-	 * ext4からfallocateシステムコールがサポートされた。
+	 * 追記。ext4からfallocateシステムコールがサポートされた。
 	 * sparse file化とは違うらしい？
 	 * ext4以外の、それをサポートしていないファイルシステムだとかなり遅くなる様子。
 	 * ext4だと4秒弱で終わるフォルダのコピーが、ntfsだと12秒はかかる。
 	 * （何もしないと6秒弱、pwriteだと8秒ほど、ftruncateは何もしない場合とほぼ同じ）
+	 * 
+	 * 追記。fallocateを使用して400GBほどコピーしてfsckしたところ、
+	 * 6.0% non-contiguous とか表示された。効果あるの？
+	 * 
+	 * 追記。ftruncateはWindowsでもあまり効果が無いっぽい。
 	*/
 	if((from_size_local > 0) && (to_open == true))
 	{
@@ -243,28 +250,36 @@ _Bool read_write(const OPArg *oparg, CPInfo *cpinfo, SDir *sdir, CPDD *cpdd)
 		)
 		{
 			static int itmp = -1;
+//			filesystem_is_ext4 = true;
 
 			// Ubuntu 11.04だとfallocateの第二引数は FALLOC_FL_KEEP_SIZE と FALLOC_FL_PUNCH_HOLE の二つ。
 			// linux/falloc.h のincludeが必要。
+/*
 			if((itmp = fallocate(ito, FALLOC_FL_KEEP_SIZE, 0, from_size_local)) != 0)
 			{
 				errno = itmp;
 				print_error("posix_fallocate", __FILE__, __LINE__, cpdd);
 			}
+*/
+			if((itmp = posix_fallocate(ito, 0, from_size_local)) != 0)
+			{
+				errno = itmp;
+				print_error("posix_fallocate", __FILE__, __LINE__, cpdd);
+			}
 		}
 		else
 		{
 			errno = 0;
+/*
 			if(ftruncate(ito, from_size_local) == -1)
 			{
 				print_error("ftruncate", __FILE__, __LINE__, cpdd);
 			}
-/*
+*/
 			if(pwrite(ito, "", 1, from_size_local - 1) == -1)
 			{
 				print_error("pwrite", __FILE__, __LINE__, cpdd);
 			}
-*/
 		}
 	}
 
@@ -459,6 +474,23 @@ LOOP_END:
 		}
 	}
 
+	if((from_stat == true) && (to_open == true))
+	{
+		ch_time_and_mod(&ito, &stat_buf, cpinfo, cpdd);
+	}
+
+/*
+ * fdatasyncするようにしたら、30倍遅くなった。
+
+	if((oparg->C != NOT) && (filesystem_is_ext4 == true))
+	{
+		if(fdatasync(ito) == -1)
+		{
+			print_error("fdatasync", __FILE__, __LINE__, cpdd);
+		}
+	}
+*/
+
 	if(from_open == true)
 	{
 		if(close(ifrom) == -1)
@@ -475,11 +507,6 @@ LOOP_END:
 		}
 	}
 
-	if((from_stat == true) && (to_open == true))
-	{
-		ch_time_and_mod(&stat_buf, cpinfo, cpdd);
-	}
-
 	if(total_read_local > 0)
 	{
 		cpinfo->from_size = from_size_local;
